package pl.info.kardas.passman.ejb;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.annotation.Resource;
import javax.annotation.security.DeclareRoles;
import javax.annotation.security.RolesAllowed;
import javax.ejb.Local;
import javax.ejb.Remote;
import javax.ejb.SessionContext;
import javax.ejb.Stateful;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

import org.jboss.ejb3.annotation.SecurityDomain;

import pl.info.kardas.passman.jpa.PassManObject;
import pl.info.kardas.passman.jpa.PassManObjectAccessLevels;
import pl.info.kardas.passman.jpa.PassManObjectAdmin;
import pl.info.kardas.passman.jpa.PassManObjectAdminGroup;
import pl.info.kardas.passman.jpa.PassManObjectGroup;
import pl.info.kardas.passman.jpa.PassManObjectAdminRights;
import pl.info.kardas.passman.jpa.PassManObjectAdminGroupRights;
import pl.info.kardas.passman.jpa.PassManObjectType;
import pl.info.kardas.passman.jpa.PassManObjectUserPassword;

/**
 * Session Bean implementation class PassManBean
 */
@Stateful
@Remote(PassManBeanRemote.class)
@Local(PassManBeanLocal.class)
@SecurityDomain("PassManSecurityDomain")
@DeclareRoles({"admin", "user"})
@RolesAllowed({"admin", "user"})
public class PassManBean implements PassManBeanRemote, PassManBeanLocal {

	@PersistenceContext(unitName="PassMan-JPA")
	protected EntityManager entityManager;
	
	@Resource 
	protected SessionContext ctx;
	
	protected PassManObjectAdmin logedUser = null;
	
    /**
     * Default constructor. 
     */
    public PassManBean() {
    	
    	// TODO Auto-generated constructor stub
    }

    /**
     * Metoda loguje użytkownika do systemu
     * @param username
     * @return
     * @throws Exception
     */
    public boolean login() throws Exception{
    	logedUser = (PassManObjectAdmin) entityManager.find(
    			PassManObjectAdmin.class, 
    			ctx.getCallerPrincipal().getName()); //Pobranie z kontekstu loginu
    	if (logedUser == null){
    		throw new Exception("No user in database: " + ctx.getCallerPrincipal().getName());
    	}
    	return true;
    }
    
    /**
     * Metoda wylogowująca użytkownika
     */
    public void logout(){
    	logedUser = null;
    }
    
    /**
     * Pobieranie listy obiektów do których użytkownik ma dostęp
     * @return
     * @throws Exception
     */
    
    @SuppressWarnings("unchecked")
	public List<PassManObject> getPassManObject(Integer firstResult, Integer maxResults) throws Exception {
    	if(logedUser == null) login();
    	Query q = entityManager.createNamedQuery("GetObjectsByLogin").setParameter("admin", logedUser).setParameter("group", logedUser.getGroups());
    	if(firstResult != null)
    		q.setFirstResult(firstResult);

    	if(maxResults != null)
    		q.setFirstResult(maxResults);

    	return (List<PassManObject>) q.getResultList();
    }

    @SuppressWarnings("unchecked")
	public List<PassManObjectUserPassword> getPassManObjectUserPassword(Integer firstResult, Integer maxResults) throws Exception {
    	if(logedUser == null) login();
    	Query q = entityManager.createNamedQuery("GetUserPasswordsByLogin").setParameter("admin", logedUser).setParameter("group", logedUser.getGroups());
    	if(firstResult != null)
    		q.setFirstResult(firstResult);

    	if(maxResults != null)
    		q.setFirstResult(maxResults);

    	return (List<PassManObjectUserPassword>) q.getResultList();
    }

    @RolesAllowed("admin")
    public void addPassManObjectAdmin(PassManObjectAdmin admin) throws Exception {
    	entityManager.persist(admin);
    }
    
    @RolesAllowed("admin")
    public void addPassManObjectAdminGroup(PassManObjectAdminGroup group ) throws Exception {
    	entityManager.persist(group);
    }
    
	public void addPassManObject(PassManObject passManObject) throws Exception {
    	if(logedUser == null) login();
    	/*
    	 * Domyślnie ustawiamy iż osoba która dodaje nowy obiekt ma możliwość RW
    	 */
    	PassManObjectAdminRights adminRights = new PassManObjectAdminRights(logedUser, PassManObjectAccessLevels.MODIFY);
    	adminRights.setAccessLevel(PassManObjectAccessLevels.MODIFY);
    	if(passManObject.getAdminRights() == null){
    		passManObject.setAdminRights(new ArrayList<PassManObjectAdminRights>());
    		entityManager.persist(adminRights);
    		passManObject.getAdminRights().add(adminRights);
    	} else {
    		Iterator<?>  persistitr = passManObject.getAdminRights().iterator();
    		while(persistitr.hasNext()){
    			PassManObjectAdminRights o = (PassManObjectAdminRights) persistitr.next();
    			entityManager.persist(o);
    		}
    	}
    	
    	if(passManObject.getAdminGroupRights() != null){
    		Iterator<?>  persistitr = passManObject.getAdminGroupRights().iterator();
    		while(persistitr.hasNext()){
    			PassManObjectAdminGroupRights o = (PassManObjectAdminGroupRights) persistitr.next();
    			entityManager.persist(o);
    		}
    	}
    	
    	entityManager.persist(passManObject);
    }

    public void addPassManObjectUserPassword(PassManObjectUserPassword userPassword) throws Exception {
    	if(logedUser == null) login();
    	/*
    	 * Domyślnie ustawiamy iż osoba która dodaje nowy obiekt ma możliwość RW
    	 */
    	PassManObjectAdminRights adminRights = new PassManObjectAdminRights(logedUser, PassManObjectAccessLevels.MODIFY);
    	adminRights.setAccessLevel(PassManObjectAccessLevels.MODIFY);
    	if(userPassword.getAdminRights() == null){
    		userPassword.setAdminRights(new ArrayList<PassManObjectAdminRights>());
    		entityManager.persist(adminRights);
    		userPassword.getAdminRights().add(adminRights);
    	} else {
    		Iterator<?>  persistitr = userPassword.getAdminRights().iterator();
    		while(persistitr.hasNext()){
    			PassManObjectAdminRights o = (PassManObjectAdminRights) persistitr.next();
    			entityManager.persist(o);
    		}
    	}
    	
    	if(userPassword.getAdminGroupRights() != null){
    		Iterator<?>  persistitr = userPassword.getAdminGroupRights().iterator();
    		while(persistitr.hasNext()){
    			PassManObjectAdminGroupRights o = (PassManObjectAdminGroupRights) persistitr.next();
    			entityManager.persist(o);
    		}
    	}

    	entityManager.persist(userPassword);
    }
    
    @RolesAllowed("admin")
    public void addPassManObjectGroup(PassManObjectGroup group) throws Exception {
    	entityManager.persist(group);
    }

    @RolesAllowed("admin")
    public void addPassManObjectType(PassManObjectType type) throws Exception {
    	entityManager.persist(type);
    }

    @SuppressWarnings("unchecked")
	public List<PassManObjectAdminGroup> getPassManObjectAdminGroups(Integer firstResult, Integer maxResults) throws Exception {
    	Query q = entityManager.createNamedQuery("ListAllAdminGroups");
    	if(firstResult != null)
    		q.setFirstResult(firstResult);

    	if(maxResults != null)
    		q.setFirstResult(maxResults);

    	return (List<PassManObjectAdminGroup>) q.getResultList();
    }
            
    public Integer getNumberOfPassManObjectAdminGroups() throws Exception {
    	return entityManager.createNamedQuery("ListAllAdminGroups").getResultList().size();
    }
    
    @SuppressWarnings("unchecked")
	public List<PassManObjectAdmin> getPassManObjectAdmins(Integer firstResult, Integer maxResults) throws Exception {
    	Query q = entityManager.createNamedQuery("ListAllAdmins");
    	if(firstResult != null)
    		q.setFirstResult(firstResult);

    	if(maxResults != null)
    		q.setFirstResult(maxResults);

    	return (List<PassManObjectAdmin>) q.getResultList();
    }

    public Integer getNumberOfPassManObjectAdmins() throws Exception {
    	return entityManager.createNamedQuery("ListAllAdmins").getResultList().size();
    }

	@SuppressWarnings("unchecked")
	public List<PassManObjectType> getPassManObjectTypes(Integer firstResult,
			Integer maxResults) throws Exception {
    	Query q = entityManager.createNamedQuery("ListAllObjectTypes");
    	if(firstResult != null)
    		q.setFirstResult(firstResult);

    	if(maxResults != null)
    		q.setFirstResult(maxResults);

    	return (List<PassManObjectType>) q.getResultList();
	}

	public Integer getNumberOfPassManObjectTypes() throws Exception {
    	return entityManager.createNamedQuery("ListAllObjectTypes").getResultList().size();
	}

	@SuppressWarnings("unchecked")
	public List<PassManObjectGroup> getPassManObjectGroups(Integer firstResult,
			Integer maxResults) throws Exception {
    	Query q = entityManager.createNamedQuery("ListAllObjectGroups");
    	if(firstResult != null)
    		q.setFirstResult(firstResult);

    	if(maxResults != null)
    		q.setFirstResult(maxResults);

    	return (List<PassManObjectGroup>) q.getResultList();
	}

	public Integer getNumberOfPassManObjectGroups() throws Exception {
    	return entityManager.createNamedQuery("ListAllObjectGroups").getResultList().size();
	}

}
