package pl.info.kardas.passman.jpa;

import static javax.persistence.GenerationType.IDENTITY;

import java.io.Serializable;
import java.lang.Integer;
import java.lang.String;
import java.util.List;

import javax.persistence.*;

/**
 * Entity implementation class for Entity: Object
 *
 */
@Entity
@Table(name = "PassManObject")
@NamedQuery(name = "GetObjectsByLogin", query = "select" +
"  o " +
" from" +
"  PassManObject as o" +   
" JOIN" +
"  o.adminRights adminRights" +   
" JOIN" +
"  o.adminGroupRights adminGroupRights" +   
" where" +
"  (" +
"   adminRights.admin = :admin" +    
"   and  (" +

"    adminRights.accessLevel = pl.info.kardas.passman.jpa.PassManObjectAccessLevels.READ_ONLY" +     
"    or adminRights.accessLevel = pl.info.kardas.passman.jpa.PassManObjectAccessLevels.MODIFY" +
"   )" +
"  )" +   
"  or (" +
"   adminGroupRights.group = :group" +    
"   and  (" +
"    adminGroupRights.accessLevel = pl.info.kardas.passman.jpa.PassManObjectAccessLevels.READ_ONLY" +     
"    or adminGroupRights.accessLevel = pl.info.kardas.passman.jpa.PassManObjectAccessLevels.MODIFY" +
"   )" + 
"  )")
public class PassManObject implements Serializable {

	private static final long serialVersionUID = 1L;
	
	@Id
	@GeneratedValue(strategy = IDENTITY)
	@Column(name = "id", unique = true, nullable = false)
	private Integer id;
	
	@Column(name = "name")
	private String name;
	
	@Column(name = "comment")
	private String comment;
	
	@ManyToMany(targetEntity = pl.info.kardas.passman.jpa.PassManObjectGroup.class)
	@JoinTable(name = "PassManObject_PassManObjectGroup")
	private List<PassManObjectGroup> groups;
	
	@ManyToOne
	private PassManObjectType type;
	/*
	 * Możliwość nadania uprawnień per Obiekt (serwer)
	 */
	@ManyToMany(targetEntity = pl.info.kardas.passman.jpa.PassManObjectAdminGroupRights.class)
	private List<PassManObjectAdminGroupRights> adminGroupRights;
	
	@ManyToMany(targetEntity = pl.info.kardas.passman.jpa.PassManObjectAdminRights.class)
	private List<PassManObjectAdminRights> adminRights;
	
	@Column(name = "hashKey")
	private String hashKey;

	@OneToMany(targetEntity = pl.info.kardas.passman.jpa.PassManObjectUserPassword.class)
	@JoinTable(name = "PassManObject_PassManObjectUserPassword")
	private List<PassManObjectUserPassword> userPasswords;

	public PassManObject() {
		super();
	}   
	public Integer getId() {
		return this.id;
	}

	public void setId(Integer id) {
		this.id = id;
	}   
	public String getName() {
		return this.name;
	}

	public void setName(String name) {
		this.name = name;
	}   
	public String getComment() {
		return this.comment;
	}

	public void setComment(String comment) {
		this.comment = comment;
	}   
	public List<PassManObjectGroup> getGroups() {
		return this.groups;
	}

	public void setGroups(List<PassManObjectGroup> groups) {
		this.groups = groups;
	}   
	public PassManObjectType getType() {
		return this.type;
	}

	public void setType(PassManObjectType type) {
		this.type = type;
	}   
	
	public String getHashKey() {
		return this.hashKey;
	}

	public void setHashKey(String hashKey) {
		this.hashKey = hashKey;
	}
	public void setUserPasswords(List<PassManObjectUserPassword> userPasswords) {
		this.userPasswords = userPasswords;
	}
	public List<PassManObjectUserPassword> getUserPasswords() {
		return userPasswords;
	}
	public void setAdminGroupRights(List<PassManObjectAdminGroupRights> adminGroupRights) {
		this.adminGroupRights = adminGroupRights;
	}
	public List<PassManObjectAdminGroupRights> getAdminGroupRights() {
		return adminGroupRights;
	}
	public void setAdminRights(List<PassManObjectAdminRights> adminRights) {
		this.adminRights = adminRights;
	}
	public List<PassManObjectAdminRights> getAdminRights() {
		return adminRights;
	}
   
}
