package pl.info.kardas.passman.jpa;

import static javax.persistence.GenerationType.IDENTITY;

import java.io.Serializable;
import java.lang.Integer;
import java.lang.String;
import java.util.List;

import javax.persistence.*;


/**
 * Entity implementation class for Entity: ObjectUserPassword
 *
 */
@Entity
@Table(name = "PassManObjectUserPassword")
@NamedQuery(name = "GetUserPasswordsByLogin", query = "select" +
"  o " +
" from" +
"  PassManObjectUserPassword as o" +   
" JOIN" +
"  o.adminRights adminRights" +   
" JOIN" +
"  o.adminGroupRights adminGroupRights" +   
" where" +
"  (" +
"   adminRights.admin = :admin" +    
"   and  (" +

"    adminRights.accessLevel = pl.info.kardas.passman.jpa.PassManObjectAccessLevels.READ_ONLY" +     
"    or adminRights.accessLevel = pl.info.kardas.passman.jpa.PassManObjectAccessLevels.MODIFY" +
"   )" +
"  )" +   
"  or (" +
"   adminGroupRights.group = :group" +    
"   and  (" +
"    adminGroupRights.accessLevel = pl.info.kardas.passman.jpa.PassManObjectAccessLevels.READ_ONLY" +     
"    or adminGroupRights.accessLevel = pl.info.kardas.passman.jpa.PassManObjectAccessLevels.MODIFY" +
"   )" + 
"  )")
public class PassManObjectUserPassword implements Serializable {

	private static final long serialVersionUID = 1L;

	@Id
	@GeneratedValue(strategy = IDENTITY)
	@Column(name = "id", unique = true, nullable = false)
	private Integer id;

	@Column(name = "name")
	private String name;

	@Column(name = "comment")
	private String comment;

	@ManyToOne
	private PassManObject object;

	@Column(name = "encryptedUser")
	private String encryptedUser;

	@Column(name = "encryptedPassword")
	private String encryptedPassword;
	
	/*
	 * Możliwość nadania uprawnień per wpis użytkownika i hasła
	 */
	@ManyToMany(targetEntity = pl.info.kardas.passman.jpa.PassManObjectAdminGroupRights.class)
	private List<PassManObjectAdminGroupRights> adminGroupRights;
	
	@ManyToMany(targetEntity = pl.info.kardas.passman.jpa.PassManObjectAdminRights.class)
	private List<PassManObjectAdminRights> adminRights;
	
	private String decrypt(String encrypted) throws Exception{
		if(object.getHashKey() == null) throw new Exception("No HashKey for object " + object.getName());
		return encrypted;
	}

	private String encrypt(String decrypted) throws Exception{
		if(object.getHashKey() == null) throw new Exception("No HashKey for object " + object.getName());
		
		return decrypted;
	}

	public PassManObjectUserPassword() {
		super();
	}   
	public Integer getId() {
		return this.id;
	}

	public void setId(Integer id) {
		this.id = id;
	}   
	public String getName() {
		return this.name;
	}

	public void setName(String name) {
		this.name = name;
	}   
	public String getComment() {
		return this.comment;
	}

	public void setComment(String comment) {
		this.comment = comment;
	}   
	public PassManObject getObject() {
		return this.object;
	}

	public void setObject(PassManObject object) {
		this.object = object;
	}   
	public String getEncryptedUser() {
		return this.encryptedUser;
	}
	public void setEncryptedUser(String encryptedUser) {
		this.encryptedUser = encryptedUser;
	}   
	public String getEncryptedPassword() {
		return this.encryptedPassword;
	}
	public void setEncryptedPassword(String encryptedPassword) {
		this.encryptedPassword = encryptedPassword;
	}
	public String getUser() throws Exception {
		return decrypt(this.encryptedUser);
	}
	public void setUser(String user) throws Exception {
		this.encryptedUser = encrypt(user);
	}   
	public String getPassword() throws Exception {
		return decrypt(this.encryptedPassword);
	}
	public void setPassword(String password) throws Exception {
		this.encryptedPassword = encrypt(password);
	}

	public void setAdminGroupRights(List<PassManObjectAdminGroupRights> adminGroupRights) {
		this.adminGroupRights = adminGroupRights;
	}

	public List<PassManObjectAdminGroupRights> getAdminGroupRights() {
		return adminGroupRights;
	}

	public void setAdminRights(List<PassManObjectAdminRights> adminRights) {
		this.adminRights = adminRights;
	}

	public List<PassManObjectAdminRights> getAdminRights() {
		return adminRights;
	}

}
